import re
import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"}

            uprn = kwargs.get("uprn")
            user_paon = kwargs.get("paon")
            postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            driver = create_webdriver(web_driver, headless)
            driver.get(kwargs.get("url"))

            wait = WebDriverWait(driver, 60)
            post_code_search = wait.until(
                EC.presence_of_element_located((By.CLASS_NAME, "form-control"))
            )
            post_code_search.send_keys(postcode)

            submit_btn = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f"//button[contains(@class, 'mx-name-actionButton3')]")
                )
            )

            submit_btn.send_keys(Keys.ENTER)

            dropdown_options = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f'//option[contains(text(), "{user_paon}")]')
                )
            )
            parent_element = dropdown_options.find_element(
                By.XPATH, ".."
            )  # Using ".." to move up to the parent element

            # Create a 'Select' for it, then select the first address in the list
            # (Index 0 is "Make a selection from the list")
            dropdownSelect = Select(parent_element)
            dropdownSelect.select_by_visible_text(str(user_paon))

            submit_btn = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f"//button[contains(@class, 'mx-name-actionButton5')]")
                )
            )

            submit_btn.send_keys(Keys.ENTER)

            results = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f'//span[contains(@class,"collection-sub")]')
                )
            )

            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            # Initialize current month and year (you can modify these values based on your requirement)
            data = {"bins": []}

            current_date = datetime.now()

            # Find all elements with class starting with 'mx-name-index-'
            bins = soup.find_all(class_=lambda x: x and x.startswith("mx-name-index-"))

            for bin_item in bins:
                bin_type = bin_item.find(class_="collection-main").text.strip()
                day_of_week_elements = bin_item.find_all(class_="collection-header")
                bin_date = None

                for elem in day_of_week_elements:
                    if (
                        elem.text.strip() != bin_type
                    ):  # Avoid taking the bin type as the date
                        next_sibling = elem.find_next_sibling()
                        if next_sibling:
                            bin_date_str = next_sibling.text.strip()
                            try:
                                # Try parsing the date string in the format 'dd Month' (e.g., '30 Dec', '5 January')
                                bin_date = datetime.strptime(bin_date_str, "%d %b")
                            except ValueError:
                                try:
                                    # If the above format fails, try 'dd MonthName' (e.g., '30 December', '5 January')
                                    bin_date = datetime.strptime(bin_date_str, "%d %B")
                                except ValueError:
                                    pass

                            if bin_date:
                                # Set the year based on the logic provided
                                if bin_date.month < current_date.month:
                                    bin_date = bin_date.replace(
                                        year=current_date.year + 1
                                    )
                                else:
                                    bin_date = bin_date.replace(year=current_date.year)
                                # Format the date to the desired format
                                bin_date = bin_date.strftime("%d/%m/%Y")
                                break
                dict_data = {"type": bin_type, "collectionDate": bin_date}
                data["bins"].append(dict_data)
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        data = {"bins": []}

        uprn = kwargs.get("uprn")
        check_uprn(uprn)

        response = requests.post(
            f"https://wastecollections.haringey.gov.uk/property/{uprn}"
        )
        if response.status_code != 200:
            raise ConnectionAbortedError("Issue encountered getting addresses.")

        soup = BeautifulSoup(response.text, features="html.parser")
        soup.prettify()

        sections = soup.find_all("div", {"class": "property-service-wrapper"})

        date_regex = re.compile(r"\d{2}/\d{2}/\d{4}")
        for section in sections:
            service = section.find("h3", {"class": "service-name"}).text
            next_collection = (
                section.find("tbody")
                .find("td", {"class": "next-service"})
                .find(text=date_regex)
            )
            # Remove Collect and Collect Paid from the start of some bin entry names
            # to make the naming more consistant.
            dict_data = {
                "type": service.replace("Collect ", "").replace("Paid ", "").strip(),
                "collectionDate": next_collection.strip(),
            }
            data["bins"].append(dict_data)

        return data



from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support.ui import Select, WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from dateutil.parser import parse

from uk_bin_collection.uk_bin_collection.common import create_webdriver, date_format
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    def wait_for_element(self, driver, element_type, element: str, timeout: int = 5):
        element_present = EC.presence_of_element_located((element_type, element))
        self.wait_for_element_conditions(driver, element_present, timeout=timeout)

    def wait_for_element_conditions(self, driver, conditions, timeout: int = 5):
        try:
            WebDriverWait(driver, timeout).until(conditions)
        except TimeoutException:
            print("Timed out waiting for page to load")
            raise

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            page = "https://westsuffolk-self.achieveservice.com/service/WSS_EX_Inf_Bin_Collection_Postcode_Lookup"

            # Assign user info
            user_postcode = kwargs.get("postcode")
            user_paon = kwargs.get("paon")

            # Create Selenium webdriver
            driver = create_webdriver(web_driver, headless)
            driver.get(page)

            # Click the cookie button
            css_selector = "#close-cookie-message"
            self.wait_for_element(driver, By.CSS_SELECTOR, css_selector)
            cookie_button = driver.find_element(By.CSS_SELECTOR, css_selector)
            cookie_button.click()

            # switch to the form iframe
            iframe = driver.find_element(By.CSS_SELECTOR, "#fillform-frame-1")
            driver.switch_to.frame(iframe)

            # Wait for form to load
            xpath_selector = '//*[@id="postcode_search"]'
            self.wait_for_element(driver, By.XPATH, xpath_selector, timeout=6)

            # Send postcode
            postcode_input_box = driver.find_element(By.XPATH, xpath_selector)
            postcode_input_box.send_keys(user_postcode)
            postcode_input_box.send_keys(Keys.ENTER)

            search_for_address_button = driver.find_element(
                By.CSS_SELECTOR, "#addresssearch"
            )
            search_for_address_button.click()

            # Find address
            self.wait_for_element(
                driver, By.CSS_SELECTOR, "#selectaddr > option:nth-child(2)"
            )
            select_address_dropdown = Select(
                driver.find_element(By.CSS_SELECTOR, "#selectaddr")
            )
            if user_paon is not None:
                for option in select_address_dropdown.options:
                    if user_paon in option.text:
                        select_address_dropdown.select_by_visible_text(option.text)
                        break
            else:
                # If we've not been supplied an address, pick the second entry
                select_address_dropdown.select_by_index(1)

            # Click the get schedule button (Once it's available)
            self.wait_for_element_conditions(
                driver,
                EC.element_to_be_clickable((By.CSS_SELECTOR, "#searchforcollections")),
            )
            check_schedules_button = driver.find_element(
                By.CSS_SELECTOR, "#searchforcollections"
            )
            check_schedules_button.click()

            # Grab the bin output data
            # Span is of the format:
            #   Your Next bin collections are:
            #   Black bin - Wednesday 29th November
            #    Blue bin - Wednesday 6th December
            #    Brown bin - Wednesday 6th December
            output_span_css_selector = "//span[@data-name='statictext6']"
            output_span_ec = EC.all_of(
                EC.presence_of_element_located((By.XPATH, output_span_css_selector)),
                EC.text_to_be_present_in_element(
                    (By.XPATH, output_span_css_selector), "bin collections"
                ),
            )
            self.wait_for_element_conditions(driver, output_span_ec)
            data_span = driver.find_element(By.XPATH, output_span_css_selector)

            data = {"bins": []}
            for item in data_span.text.splitlines():
                if "bin" in item and " - " in item:
                    bin_data = item.split(" - ")
                    bin_name = bin_data[0]
                    bin_date = bin_data[1]
                    parsed_bin_date = parse(bin_date, fuzzy_with_tokens=True)[0]

                    dict_data = {
                        "type": bin_name,
                        "collectionDate": parsed_bin_date.strftime(date_format),
                    }

                    data["bins"].append(dict_data)

            # Quit Selenium webdriver to release session

        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from datetime import timedelta

from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Get page with BS4
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        # Work out some date bounds
        today = datetime.today()
        eight_weeks = datetime.today() + timedelta(days=8 * 7)
        data = {"bins": []}

        # Each month calendar is a table, so get the object then find all rows in that object.
        # Month and year is also a row and not included in the date, so save it then remove the row
        for month in soup.select('table[class*="table table-condensed"]'):
            info = month.find_all("tr")
            month_year = info[0].text.strip()
            info.pop(0)
            # Each remaining item is a bin collection, so get the type and tidy up the date.
            for item in info:
                bin_type = item.text.split(",")[0].strip()
                bin_date = datetime.strptime(
                    remove_ordinal_indicator_from_date_string(
                        item.text.split(",")[1].strip() + " " + month_year
                    ),
                    "%A %d %B %Y",
                )
                # Only include dates on or after today, but also only within eight weeks
                if (
                    today.date() <= bin_date.date() <= eight_weeks.date()
                    and "cancelled" not in bin_type
                ):
                    dict_data = {
                        "type": bin_type,
                        "collectionDate": bin_date.strftime(date_format),
                    }
                    data["bins"].append(dict_data)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass
from dateutil.relativedelta import relativedelta


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Add in some variables we need
        data = {"bins": []}
        collections = []
        curr_date = datetime.today()

        # Parse the page
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        # Get all table rows on the page - enumerate gives us an index, which is handy for to keep a row count.
        # In this case, the first (0th) row is headings, so we can skip it, then parse the other data.
        for idx, row in enumerate(soup.find_all("tr")):
            if idx == 0:
                continue
            row_type = row.find("th").text.strip()
            row_data = row.find_all("td")
            # When we get the row data, we can loop through it all and parse it to datetime. Because there are no
            # years, we must add it in, then check if we need to overflow it to the following year.
            for item in row_data:
                if item.text.strip() == "Today":
                    collections.append((row_type, curr_date))
                else:
                    bin_date = datetime.strptime(
                        remove_ordinal_indicator_from_date_string(item.text.strip()),
                        "%A, %d %B",
                    ).replace(year=curr_date.year)
                    if curr_date.month == 12 and bin_date.month == 1:
                        bin_date = bin_date + relativedelta(years=1)
                    collections.append((row_type, bin_date))

        # Sort the text and list elements by date
        ordered_data = sorted(collections, key=lambda x: x[1])

        # Put the elements into the dictionary
        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1].strftime(date_format),
            }
            data["bins"].append(dict_data)

        return data



import re
import requests
from bs4 import BeautifulSoup

from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

# This script pulls (in one hit) the data from Bromley Council Bins Data
import datetime
from datetime import datetime
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.common.keys import Keys
import time


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        driver = None
        try:
            data = {"bins": []}
            headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"}

            uprn = kwargs.get("uprn")
            postcode = kwargs.get("postcode")
            web_driver = kwargs.get("web_driver")
            headless = kwargs.get("headless")
            driver = create_webdriver(web_driver, headless)
            driver.get(kwargs.get("url"))

            wait = WebDriverWait(driver, 60)
            post_code_search = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f"//input[contains(@class, 'searchAddress')]")
                )
            )
            post_code_search.send_keys(postcode)

            submit_btn = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f"//button[contains(@class, 'searchAddressButton')]")
                )
            )

            submit_btn.send_keys(Keys.ENTER)

            address_link = wait.until(
                EC.presence_of_element_located(
                    (By.XPATH, f'//a[contains(@data-uprn,"{uprn}")]')
                )
            )

            address_link.send_keys(Keys.ENTER)

            address_results = wait.until(
                EC.presence_of_element_located(
                    (By.CLASS_NAME, "your-collection-schedule-container")
                )
            )

            # Make a BS4 object
            soup = BeautifulSoup(driver.page_source, features="html.parser")
            data = {"bins": []}

            # Get the current month and year
            current_month = datetime.now().month
            current_year = datetime.now().year

            # Function to extract collection data
            def extract_collection_data(collection_div, collection_type):
                if collection_div:
                    date_element = (
                        collection_div.find(class_="recycling-collection-day-numeric")
                        or collection_div.find(class_="refuse-collection-day-numeric")
                        or collection_div.find(class_="garden-collection-day-numeric")
                    )
                    month_element = (
                        collection_div.find(class_="recycling-collection-month")
                        or collection_div.find(class_="refuse-collection-month")
                        or collection_div.find(class_="garden-collection-month")
                    )

                    if date_element and month_element:
                        collection_date = date_element.get_text(strip=True)
                        collection_month = month_element.get_text(strip=True)

                        # Combine month, date, and year into a string
                        date_string = (
                            f"{collection_date} {collection_month} {current_year}"
                        )

                        try:
                            # Convert the date string to a datetime object
                            collection_date_obj = datetime.strptime(
                                date_string, "%d %B %Y"
                            )

                            # Check if the month is ahead of the current month
                            if collection_date_obj.month >= current_month:
                                # If the month is ahead, use the current year
                                formatted_date = collection_date_obj.strftime(
                                    date_format
                                )
                            else:
                                # If the month is before the current month, use the next year
                                formatted_date = collection_date_obj.replace(
                                    year=current_year + 1
                                ).strftime(date_format)
                            # Create a dictionary for each collection entry
                            dict_data = {
                                "type": collection_type,
                                "collectionDate": formatted_date,
                            }

                            # Append dictionary data to the 'bins' list in the 'data' dictionary
                            data["bins"].append(dict_data)

                        except ValueError as e:
                            # Handle the case where the date format is invalid
                            formatted_date = "Invalid Date Format"

            # Extract Recycling collection data
            extract_collection_data(
                soup.find(
                    class_="container-fluid RegularCollectionDay"
                ).find_next_sibling("div"),
                "Recycling",
            )

            # Extract Refuse collection data
            for refuse_div in soup.find_all(
                class_="container-fluid RegularCollectionDay"
            ):
                extract_collection_data(refuse_div, "Refuse")

            # Extract Garden Waste collection data
            extract_collection_data(
                soup.find(class_="container-fluid gardenwasteCollectionDay"),
                "Garden Waste",
            )

            # Print the extracted data
        except Exception as e:
            # Here you can log the exception if needed
            print(f"An error occurred: {e}")
            # Optionally, re-raise the exception if you want it to propagate
            raise
        finally:
            # This block ensures that the driver is closed regardless of an exception
            if driver:
                driver.quit()
        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        user_postcode = kwargs.get("postcode")
        check_postcode(user_postcode)
        user_uprn = kwargs.get("uprn")
        check_uprn(user_uprn)

        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/87.0.4280.141 Safari/537.36"
        }

        requests.packages.urllib3.disable_warnings()
        with requests.Session() as s:
            # Set Headers
            s.headers = headers

            # Get the first page - This is the Search for property by Post Code page
            resource = s.get(
                "https://iweb.itouchvision.com/portal/f?p=customer:BIN_DAYS:::NO:RP:UID:13353F039C4B1454827EE05536414091A8C058F4"
            )
            # Create a BeautifulSoup object from the page's HTML
            soup = BeautifulSoup(resource.text, "html.parser")

            # The page contains a number of values that must be passed into subsequent requests - extract them here
            payload = {
                i["name"]: i.get("value", "") for i in soup.select("input[name]")
            }
            payload2 = {
                i["data-for"]: i.get("value", "")
                for i in soup.select("input[data-for]")
            }
            payload_salt = soup.select_one('input[id="pSalt"]').get("value")
            payload_protected = soup.select_one('input[id="pPageItemsProtected"]').get(
                "value"
            )

            # Add the PostCode and 'SEARCH' to the payload
            payload["p_request"] = "SEARCH"
            payload["P153_POST_CODE"] = user_postcode

            # Manipulate the lists and build the JSON that must be submitted in further requests - some data is nested
            merged_list = {**payload, **payload2}
            new_list = []
            other_list = {}
            for key in merged_list.keys():
                temp_list = {}
                val = merged_list[key]
                if key in [
                    "P153_UPRN",
                    "P153_TEMP",
                    "P153_SYSDATE",
                    "P0_LANGUAGE",
                    "P153_POST_CODE",
                ]:
                    temp_list = {"n": key, "v": val}
                    new_list.append(temp_list)
                elif key in [
                    "p_flow_id",
                    "p_flow_step_id",
                    "p_instance",
                    "p_page_submission_id",
                    "p_request",
                    "p_reload_on_submit",
                ]:
                    other_list[key] = val
                else:
                    temp_list = {"n": key, "v": "", "ck": val}
                    new_list.append(temp_list)

            json_builder = {
                "pageItems": {
                    "itemsToSubmit": new_list,
                    "protected": payload_protected,
                    "rowVersion": "",
                    "formRegionChecksums": [],
                },
                "salt": payload_salt,
            }
            json_object = json.dumps(json_builder, separators=(",", ":"))
            other_list["p_json"] = json_object

            # Set Referrer header
            s.headers.update(
                {
                    "referer": "https://iweb.itouchvision.com/portal/f?p=customer:BIN_DAYS:::NO:RP:UID:13353F039C4B1454827EE05536414091A8C058F4"
                }
            )

            # Generate POST including all the JSON we just built
            s.post(
                "https://iweb.itouchvision.com/portal/wwv_flow.accept", data=other_list
            )

            # The second page on the portal would normally allow you to select your property from a dropdown list of
            # those that are at the postcode entered on the previous page
            # The required cookies are stored within the session so re-use the session to keep them
            resource = s.get(
                "https://iweb.itouchvision.com/portal/itouchvision/r/customer/bin_days"
            )

            # Create a BeautifulSoup object from the page's HTML
            soup = BeautifulSoup(resource.text, "html.parser")

            # The page contains a number of values that must be passed into subsequent requests - extract them here
            payload = {
                i["name"]: i.get("value", "") for i in soup.select("input[name]")
            }
            payload2 = {
                i["data-for"]: i.get("value", "")
                for i in soup.select("input[data-for]")
            }
            payload_salt = soup.select_one('input[id="pSalt"]').get("value")
            payload_protected = soup.select_one('input[id="pPageItemsProtected"]').get(
                "value"
            )

            # Add the UPRN and 'SUBMIT' to the payload
            payload["p_request"] = "SUBMIT"
            payload["P153_UPRN"] = user_uprn

            # Manipulate the lists and build the JSON that must be submitted in further requests - some data is nested
            merged_list = {**payload, **payload2}
            new_list = []
            other_list = {}
            for key in merged_list.keys():
                temp_list = {}
                val = merged_list[key]
                if key in ["P153_UPRN", "P153_TEMP", "P153_SYSDATE", "P0_LANGUAGE"]:
                    temp_list = {"n": key, "v": val}
                    new_list.append(temp_list)
                elif key in ["P153_ZABY"]:
                    temp_list = {"n": key, "v": "1", "ck": val}
                    new_list.append(temp_list)
                elif key in ["P153_POST_CODE"]:
                    temp_list = {"n": key, "v": user_postcode, "ck": val}
                    new_list.append(temp_list)
                elif key in [
                    "p_flow_id",
                    "p_flow_step_id",
                    "p_instance",
                    "p_page_submission_id",
                    "p_request",
                    "p_reload_on_submit",
                ]:
                    other_list[key] = val
                else:
                    temp_list = {"n": key, "v": "", "ck": val}
                    new_list.append(temp_list)

            json_builder = {
                "pageItems": {
                    "itemsToSubmit": new_list,
                    "protected": payload_protected,
                    "rowVersion": "",
                    "formRegionChecksums": [],
                },
                "salt": payload_salt,
            }

            json_object = json.dumps(json_builder, separators=(",", ":"))
            other_list["p_json"] = json_object

            # Generate POST including all the JSON we just built
            s.post(
                "https://iweb.itouchvision.com/portal/wwv_flow.accept", data=other_list
            )

            # The third and final page on the portal shows the detail of the waste collection services
            # The required cookies are stored within the session so re-use the session to keep them
            resource = s.get(
                "https://iweb.itouchvision.com/portal/itouchvision/r/customer/bin_days"
            )

            # Create a BeautifulSoup object from the page's HTML
            soup = BeautifulSoup(resource.text, "html.parser")
            data = {"bins": []}

            # Loop through the items on the page and build a JSON object for ingestion
            for item in soup.select(".t-MediaList-item"):
                for value in item.select(".t-MediaList-body"):
                    dict_data = {
                        "type": value.select("span")[1].get_text(strip=True).title(),
                        "collectionDate": datetime.strptime(
                            value.select(".t-MediaList-desc")[0].get_text(strip=True),
                            "%A, %d %B, %Y",
                        ).strftime(date_format),
                    }
                    data["bins"].append(dict_data)

            return data
